# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: audios.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x0c\x61udios.proto\"\x18\n\x07Request\x12\r\n\x05\x64\x61tas\x18\x01 \x01(\x0c\"\x16\n\x05Reply\x12\r\n\x05\x64\x61tas\x18\x01 \x01(\t21\n\nMainServer\x12#\n\tgetStream\x12\x08.Request\x1a\x06.Reply\"\x00(\x01\x30\x01\x62\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'audios_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _REQUEST._serialized_start=16
  _REQUEST._serialized_end=40
  _REPLY._serialized_start=42
  _REPLY._serialized_end=64
  _MAINSERVER._serialized_start=66
  _MAINSERVER._serialized_end=115
# @@protoc_insertion_point(module_scope)
